<?php
declare(strict_types=1);

namespace FiloBlu\Refilo\Remote;

use Magento\Framework\App\ResourceConnection;
use Magento\Framework\DB\Ddl\Table;
use Zend_Db_Exception;
use Zend_Db_Expr;

/**
 *
 */
class BasesSqlDeletedItemFinder implements DeletedItemFinderInterface
{
    /**
     * @var string
     */
    private $table;

    /**
     * @var string
     */
    private $idColumn;

    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * @param ResourceConnection $resourceConnection
     * @param string $table
     * @param string $idColumn
     */
    public function __construct(ResourceConnection $resourceConnection,
                                string             $table,
                                string             $idColumn)
    {

        $this->table = $table;
        $this->idColumn = $idColumn;
        $this->resourceConnection = $resourceConnection;
    }

    /**
     * @return array
     * @throws Zend_Db_Exception
     */
    public function getDeletedIds(): array
    {
        $connection = $this->resourceConnection->getConnection();
        $mainTable = $connection->getTableName($this->table);

        $row = $connection->fetchRow($connection->select()->from($mainTable, [
            'max'   => new Zend_Db_Expr("MAX($this->idColumn)"),
            'total' => new Zend_Db_Expr("COUNT($this->idColumn)")
        ]));

        $temporaryTable = $this->createTemporaryTable();

        $connection->insertMultiple(
            $temporaryTable->getName(),
            array_fill(0, (int)$row['max'], ['entity_id' => null])
        );

        $idColumn = $this->idColumn;

        $queryStart = $connection
            ->select()
            ->from(['l' => $temporaryTable->getName()], [$idColumn])
            ->joinLeft(['r' => $mainTable], "l.$idColumn = r.$idColumn", [])
            ->where("r.$idColumn IS NULL");

        $output = $connection->fetchCol($queryStart) ?? [];

        $connection->dropTable($temporaryTable->getName());

        return $output;
    }

    /**
     * @return Table
     * @throws Zend_Db_Exception
     */
    public function createTemporaryTable(): Table
    {
        $connection = $this->resourceConnection->getConnection();
        $tableName = $this->table . '_' . str_replace('.', '_', uniqid('tmp', true));
        $table = $connection->newTable($tableName);
        $table->addColumn(
            $this->idColumn,
            Table::TYPE_INTEGER,
            10,
            ['unsigned' => true, 'nullable' => false, 'primary' => true, 'auto_increment' => true],
            'Entity ID'
        );

        $table->setOption('type', 'memory');
        $connection->createTemporaryTable($table);
        return $table;
    }
}
